@@ -111,10 +111,11 @@ int listenfd = -1;
 static int ctrlfd = -1;
 static char *jail = NULL;
 static char *user = NULL;
-static char *proto = "tcp";
 
 static char *dsr_if, *dsr_ip;
 
+struct sockaddr_storage *source = NULL;
+
 #ifdef WINDOWS
 /* because Windows scribbles over errno in an uncalled-for manner */
 static int saved_errno;
@@ -929,6 +930,7 @@ static void init(int argc, char **argv)
 {
 	int i;
 	int server;
+	int proto;
 
 	DEBUG(2, "init(%d, %p); port = %d", argc, argv, port);
 
@@ -943,6 +945,9 @@ static void init(int argc, char **argv)
 
 	server = 0;
 
+	if (udp) proto = SOCK_DGRAM;
+	else proto = SOCK_STREAM;
+
 	for (i = 1; i < argc; i++) {
 		DEBUG(2, "server[%d] = %s", server, argv[i]);
 		expand_servertable(server+1);
@@ -1053,7 +1058,7 @@ static int open_unix_listener(char *a)
 }
 #endif
 
-static int open_listener(char *a)
+static int open_listener(char *a, int proto)
 {
 	int listenfd;
 	struct sockaddr_storage ss;
@@ -1095,7 +1100,7 @@ static int open_listener(char *a)
 	}
 	pen_setport(&ss, port);
 
-	listenfd = socket_nb(ss.ss_family, protoid, 0);
+	listenfd = socket_nb(ss.ss_family, proto, 0);
 	DEBUG(2, "local address=[%s:%d]", b, port);
 
 	setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, (void *)&one, sizeof one);
@@ -1374,7 +1379,7 @@ static void do_cmd(char *b, void (*output)(void *, char *, ...), void *op)
 				n = close(listenfd);
 				DEBUG(2, "close(listenfd=%d) returns %d", listenfd, n);
 			}
-			listenfd = open_listener(p);
+			listenfd = open_listener(p, udp ? SOCK_DGRAM : SOCK_STREAM);
 			/* we may need to defer this if we haven't called event_init yet */
 			if (event_add) event_add(listenfd, EVENT_READ);
 			DEBUG(2, "new listenfd = %d", listenfd);
@@ -1508,6 +1513,15 @@ static void do_cmd(char *b, void (*output)(void *, char *, ...), void *op)
 		int fd = p ? atoi(p) : 0;
 		int conn = fd2conn_get(fd);
 		output(op, "Socket %d belongs to connection %d\n", fd, conn);
+	} else if (!strcmp(p, "source")) {
+		p = strtok(NULL, " ");
+		if (!source) source = pen_malloc(sizeof *source);
+		if (pen_aton(p, source) == 0) {
+			debug("pen_aton(%d, source) returns 0", p);
+			output(op, "unable to set source address to '%s'", p);
+			free(source);
+			source = NULL;
+		}
 	} else if (!strcmp(p, "status")) {
 		if (webstats()) {
 			fp = fopen(webfile, "r");
@@ -1728,7 +1742,7 @@ static void add_client(int downfd, struct sockaddr_storage *cli_addr)
 			return;
 		}
 	/* we need a downfd for udp as well */
-		downfd = socket_nb(cli_addr->ss_family, protoid, 0);
+		downfd = socket_nb(cli_addr->ss_family, udp ? SOCK_DGRAM : SOCK_STREAM, 0);
 		if (downfd == -1) {
 			debug("Can't create downfd");
 			return;
@@ -2492,17 +2506,12 @@ int main(int argc, char **argv)
 		if (getuid() == 0 && user == NULL) {
 			debug("Won't open control port running as root; use -u to run as different user");
 		} else {
-			ctrlfd = open_listener(ctrlport);
+			ctrlfd = open_listener(ctrlport, SOCK_STREAM);
 		}
 	}
 
 
 	/* Balancing port */
-	if (udp) {
-		protoid = SOCK_DGRAM;
-		proto = "udp";
-	}
-
 	if (listenfd == -1) {
 		snprintf(listenport, sizeof listenport, "%s", argv[0]);
 		/* Direct server return */
@@ -2512,7 +2521,7 @@ int main(int argc, char **argv)
 				error("Can't initialize direct server return");
 			}
 		} else {
-			listenfd = open_listener(listenport);
+			listenfd = open_listener(listenport, udp ? SOCK_DGRAM : SOCK_STREAM);
 		}
 	}
 	init(argc, argv);
